from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import (StripeTestCase)


TEST_RESOURCE_ID = 'rp_123'


class RecipientTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.Recipient.list()
        self.assert_requested(
            'get',
            '/v1/recipients'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Recipient)

    def test_is_retrievable(self):
        resource = stripe.Recipient.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/recipients/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Recipient)

    def test_is_creatable(self):
        resource = stripe.Recipient.create(
            type='individual',
            name='NAME'
        )
        self.assert_requested(
            'post',
            '/v1/recipients'
        )
        self.assertIsInstance(resource, stripe.Recipient)

    def test_is_saveable(self):
        resource = stripe.Recipient.retrieve(TEST_RESOURCE_ID)
        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/recipients/%s' % resource.id
        )

    def test_is_modifiable(self):
        resource = stripe.Recipient.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/recipients/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.Recipient)

    def test_is_deletable(self):
        resource = stripe.Recipient.retrieve(TEST_RESOURCE_ID)
        # Unfortunately stripe-mock will return a resource with a different
        # ID, so we need to store the original ID for the request assertion
        resource_id = resource.id
        resource.delete()
        self.assert_requested(
            'delete',
            '/v1/recipients/%s' % resource_id
        )
        self.assertTrue(resource.deleted)

    def test_can_list_transfers(self):
        recipient = stripe.Recipient.retrieve(TEST_RESOURCE_ID)
        resources = recipient.transfers()
        self.assert_requested(
            'get',
            '/v1/transfers',
            {'recipient': recipient.id}
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.Transfer)



from __future__ import absolute_import, division, print_function

from stripe import util
from stripe.api_resources.abstract import CreateableAPIResource
from stripe.api_resources.abstract import UpdateableAPIResource
from stripe.api_resources.abstract import ListableAPIResource


class PaymentIntent(CreateableAPIResource, UpdateableAPIResource,
                    ListableAPIResource):
    OBJECT_NAME = 'payment_intent'

    @classmethod
    def class_name(cls):
        return 'payment_intent'

    def cancel(self, idempotency_key=None, **params):
        url = self.instance_url() + '/cancel'
        headers = util.populate_headers(idempotency_key)
        self.refresh_from(self.request('post', url, params, headers))
        return self

    def capture(self, idempotency_key=None, **params):
        url = self.instance_url() + '/capture'
        headers = util.populate_headers(idempotency_key)
        self.refresh_from(self.request('post', url, params, headers))
        return self

    def confirm(self, idempotency_key=None, **params):
        url = self.instance_url() + '/confirm'
        headers = util.populate_headers(idempotency_key)
        self.refresh_from(self.request('post', url, params, headers))
        return self



from __future__ import absolute_import, division, print_function

from mock import patch, ANY

import stripe
from stripe import six
from stripe.stripe_response import StripeResponse


class RequestMock(object):
    def __init__(self):
        self._real_request = stripe.api_requestor.APIRequestor.request
        self._stub_request_handler = StubRequestHandler()

    def start(self):
        self.constructor_patcher = patch(
            'stripe.api_requestor.APIRequestor.__init__',
            side_effect=stripe.api_requestor.APIRequestor.__init__,
            autospec=True)
        self.constructor_spy = self.constructor_patcher.start()

        self.request_patcher = patch(
            'stripe.api_requestor.APIRequestor.request',
            side_effect=self._patched_request,
            autospec=True)
        self.request_spy = self.request_patcher.start()

    def stop(self):
        self.request_patcher.stop()
        self.constructor_patcher.stop()

    def _patched_request(self, requestor, method, url, *args, **kwargs):
        response = self._stub_request_handler.get_response(method, url)
        if response is not None:
            return response, stripe.api_key

        return self._real_request(requestor, method, url, *args, **kwargs)

    def stub_request(self, method, url, rbody={}, rcode=200, rheaders={}):
        self._stub_request_handler.register(method, url, rbody, rcode,
                                            rheaders)

    def assert_api_version(self, expected_api_version):
        # Note that this method only checks that an API version was provided
        # as a keyword argument in APIRequestor's constructor, not as a
        # positional argument.

        if 'api_version' not in self.constructor_spy.call_args[1]:
            msg = ("Expected APIRequestor to have been constructed with "
                   "api_version='%s'. No API version was provided." %
                   expected_api_version)
            raise AssertionError(msg)

        actual_api_version = self.constructor_spy.call_args[1]['api_version']
        if actual_api_version != expected_api_version:
            msg = ("Expected APIRequestor to have been constructed with "
                   "api_version='%s'. Constructed with api_version='%s' "
                   "instead." % (expected_api_version, actual_api_version))
            raise AssertionError(msg)

    def assert_requested(self, method, url, params=ANY, headers=ANY):
        called = False
        exception = None

        # Sadly, ANY does not match a missing optional argument, so we
        # check all the possible signatures of the request method
        possible_called_args = [
            (ANY, method, url),
            (ANY, method, url, params),
            (ANY, method, url, params, headers),
        ]

        for args in possible_called_args:
            try:
                self.request_spy.assert_called_with(*args)
            except AssertionError as e:
                exception = e
            else:
                called = True
                break

        if not called:
            raise exception

    def assert_no_request(self):
        if self.request_spy.call_count != 0:
            msg = ("Expected 'request' to not have been called. "
                   "Called %s times." %
                   (self.request_spy.call_count))
            raise AssertionError(msg)

    def reset_mock(self):
        self.request_spy.reset_mock()


class StubRequestHandler(object):
    def __init__(self):
        self._entries = {}

    def register(self, method, url, rbody={}, rcode=200, rheaders={}):
        self._entries[(method, url)] = (rbody, rcode, rheaders)

    def get_response(self, method, url):
        if (method, url) in self._entries:
            rbody, rcode, rheaders = self._entries.pop((method, url))
            if not isinstance(rbody, six.string_types):
                rbody = stripe.util.json.dumps(rbody)
            stripe_response = StripeResponse(rbody, rcode, rheaders)
            return stripe_response

        return None



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'pi_123'


class PaymentIntentTest(StripeTestCase):
    FIXTURE = {
        'id': TEST_RESOURCE_ID,
        'object': 'payment_intent',
        'metadata': {},
    }

    def test_is_listable(self):
        self.stub_request(
            'get',
            '/v1/payment_intents',
            {
                'object': 'list',
                'data': [self.FIXTURE],
            }
        )

        resources = stripe.PaymentIntent.list()
        self.assert_requested(
            'get',
            '/v1/payment_intents'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.PaymentIntent)

    def test_is_retrievable(self):
        self.stub_request(
            'get',
            '/v1/payment_intents/%s' % TEST_RESOURCE_ID,
            self.FIXTURE
        )

        resource = stripe.PaymentIntent.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/payment_intents/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.PaymentIntent)

    def test_is_creatable(self):
        self.stub_request(
            'post',
            '/v1/payment_intents',
            self.FIXTURE
        )

        resource = stripe.PaymentIntent.create(
            allowed_source_types=['card'],
            amount='1234',
            currency='amount'
        )
        self.assert_requested(
            'post',
            '/v1/payment_intents',
        )
        self.assertIsInstance(resource, stripe.PaymentIntent)

    def test_is_modifiable(self):
        self.stub_request(
            'post',
            '/v1/payment_intents/%s' % TEST_RESOURCE_ID,
            self.FIXTURE
        )

        resource = stripe.PaymentIntent.modify(
            TEST_RESOURCE_ID,
            metadata={'key': 'value'}
        )
        self.assert_requested(
            'post',
            '/v1/payment_intents/%s' % TEST_RESOURCE_ID,
            {'metadata': {'key': 'value'}}
        )
        self.assertIsInstance(resource, stripe.PaymentIntent)

    def test_is_saveable(self):
        resource = stripe.PaymentIntent.construct_from(self.FIXTURE,
                                                       stripe.api_key)

        self.stub_request(
            'post',
            '/v1/payment_intents/%s' % TEST_RESOURCE_ID,
            self.FIXTURE
        )

        resource.metadata['key'] = 'value'
        resource.save()
        self.assert_requested(
            'post',
            '/v1/payment_intents/%s' % resource.id,
            {'metadata': {'key': 'value'}}
        )
        self.assertIsInstance(resource, stripe.PaymentIntent)

    def test_can_cancel(self):
        resource = stripe.PaymentIntent.construct_from(self.FIXTURE,
                                                       stripe.api_key)

        self.stub_request(
            'post',
            '/v1/payment_intents/%s/cancel' % TEST_RESOURCE_ID,
            self.FIXTURE
        )

        resource.cancel()
        self.assert_requested(
            'post',
            '/v1/payment_intents/%s/cancel' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.PaymentIntent)

    def test_can_capture(self):
        resource = stripe.PaymentIntent.construct_from(self.FIXTURE,
                                                       stripe.api_key)

        self.stub_request(
            'post',
            '/v1/payment_intents/%s/capture' % TEST_RESOURCE_ID,
            self.FIXTURE
        )

        resource.capture()
        self.assert_requested(
            'post',
            '/v1/payment_intents/%s/capture' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.PaymentIntent)

    def test_can_confirm(self):
        resource = stripe.PaymentIntent.construct_from(self.FIXTURE,
                                                       stripe.api_key)

        self.stub_request(
            'post',
            '/v1/payment_intents/%s/confirm' % TEST_RESOURCE_ID,
            self.FIXTURE
        )

        resource.confirm()
        self.assert_requested(
            'post',
            '/v1/payment_intents/%s/confirm' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.PaymentIntent)



from __future__ import absolute_import, division, print_function

import stripe
from tests.helper import StripeTestCase


TEST_RESOURCE_ID = 'usd'


class ExchangeRateTest(StripeTestCase):
    def test_is_listable(self):
        resources = stripe.ExchangeRate.list()
        self.assert_requested(
            'get',
            '/v1/exchange_rates'
        )
        self.assertIsInstance(resources.data, list)
        self.assertIsInstance(resources.data[0], stripe.ExchangeRate)

    def test_is_retrievable(self):
        resource = stripe.ExchangeRate.retrieve(TEST_RESOURCE_ID)
        self.assert_requested(
            'get',
            '/v1/exchange_rates/%s' % TEST_RESOURCE_ID
        )
        self.assertIsInstance(resource, stripe.ExchangeRate)
